class StdClass(dict):
    """
    PHP like stdClass
    Can be used as a dict or as an object
    """
    __getattr__ = dict.get
    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__


class DictObject(dict):
    """
    Class for storing data in dict
    """

    # def __init__(self, *args, **kwargs):
    #     super(DictObject, self).__init__(*args, **kwargs)

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        """
        Return string representation of dict
        """
        # return [f for f in dir(self) if not callable(getattr(self, f))]
        return self.__dict__


def obj(d: dict) -> DictObject:
    """
    Convert dict to DictObject
    """
    for k in d:
        if isinstance(d[k], dict):
            d[k] = obj(d[k])

    return type("DictObject", (DictObject,), d)


def convert_class_to_dict(obj: callable, prefix: str | None = None) -> dict:
    """
    Convert class to dict
    """
    if prefix is None:
        prefix = obj().__class__.__name__.upper() + "_"
    return {
        prefix + k.upper(): v
        for k, v in obj.__dict__.items()
        if not k.startswith("_")
    }



class ShortLambda:
    """
    Short lambda realisation
    """

    def __ne__(self, obj) -> callable[[object], bool]: return lambda x: x != obj

    def __eq__(self, obj) -> callable[[object], bool]: return lambda x: x == obj

    def __gt__(self, obj) -> callable[[object], bool]: return lambda x: x > obj

    def __ge__(self, obj) -> callable[[object], bool]: return lambda x: x >= obj

    def __lt__(self, obj) -> callable[[object], bool]: return lambda x: x < obj

    def __le__(self, obj) -> callable[[object], bool]: return lambda x: x <= obj


class X(ShortLambda):
    """
    Alias for ShortLambda class
    """
    ...



def singleton(clas):
    """
    Decorator to make a class a singleton.
    """
    instances = {}

    def getinstance(*args, **kwargs):
        """
        Get an instance of the singleton.
        """
        if clas not in instances:
            instances[clas] = clas(*args, **kwargs)
        return instances[clas]

    return getinstance



#!/usr/bin/env python
import sys
import pathlib
import tomllib
from setuptools import find_packages, setup

WORK_DIR = pathlib.Path(__file__).parent

# Check python version
MINIMAL_PY_VERSION = (3, 10, 8)
if sys.version_info < MINIMAL_PY_VERSION:
    raise RuntimeError("aiogram works only with Python {}+".format(".".join(map(str, MINIMAL_PY_VERSION))))


def get_description() -> str:
    """
    Read full description from "README.rst"
    :return: description
    :rtype: str
    """
    with open("README.md", "r", encoding="utf-8") as f:
        return f.read()


def get_meta_info() -> dict:
    try:
        with open("pyproject.toml", "rb") as f:
            return tomllib.load(f)
    except IndexError:
        raise RuntimeError('Unable to determine version.')


project_meta = get_meta_info()
project_dir = project_meta["project"]["name"].replace("-", "_")

setup(
    license="MIT",
    name=project_meta["project"]["name"],
    version=project_meta["project"]["version"],
    author=project_meta["project"]["authors"][0]["name"],
    author_email=project_meta["project"]["authors"][0]["email"],
    description="A simple library for easy work with key-value storages and document databases",
    long_description=get_description(),
    long_description_content_type="text/markdown",
    url="https://github.com/frontdevops/geekjob-python-helpers",
    download_url="https://github.com/frontdevops/geekjob-python-helpers/archive/refs/tags/1.0.0.tar.gz",
    project_urls={
        "Documentation": "https://github.com/frontdevops/geekjob-python-helpers/blob/main/README.md",
        "Source": "https://github.com/frontdevops/geekjob-python-helpers",
        "Bug Tracker": "https://github.com/frontdevops/geekjob-python-helpers/issues",
    },
    keywords=["aiogram", "telegram", "storage"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10.8",
        "Programming Language :: Python :: 3.11",
        "Topic :: Software Development :: Libraries :: Application Utilities",
    ],
    package_dir={"": project_dir},
    packages=find_packages(where=project_dir,
                           exclude=("tests", "tests.*", "examples.*", "docs",),
                           ),
    include_package_data=False,
    python_requires=">=3.10.8",
    extras_require={
        "dev": [
            "pytest >= 3.7",
            "check-manifest",
            "twine"
        ]
    },
)



import os
from magic_config import Config


def _import_(fn: str) -> None:
    """
    Import module from path
    """
    if fn.startswith("_") or not fn.endswith(".py"):
        return None
    fn = fn[:-3].replace("/", ".")
    if Config.DEBUG:
        print("✅  import(" + fn + ")")
    return __import__(fn)


def recursive_import(path: str) -> None:
    """
    Import all modules from path
    """
    for fs_name in os.listdir(path):
        if fs_name.startswith("_"):
            continue
        pathname = f"{path}/{fs_name}"
        if os.path.isdir(pathname):
            recursive_import(pathname)
        else:
            _import_(pathname)
